!======================================================================== 
      subroutine  numgs (n, r_point, c_ind, a, ilu, ilup, lu,   &
              lstrow, ierr)

!======================================================================== 
!
! Numerical factorization with given sparsity pattern identified in
! the MSR data structure.  There are few safeguards in this, and
! in particular there is no tolerance test for when to reject a pivot.
! Because of the MSR storage format, the method used is a deferred update
! version.  At the k-th step, the updates from the previous rows m,
! m < k and m corresponding to a nonzero in row k of L/U, are applied
! to row k.  
!
!======================================================================== 
!
        implicit none
        integer  n, c_ind(*), r_point(*), ilu(*), ilup(*)
        double precision ::   a(*), lu(*)
        integer  lstrow(n), ierr, istart, lp

        double precision ::   li, dk
        integer  jcol, jli, i, j, k, k1

! Divide by zero tolerance:
        double precision :: SMALL
        SMALL = sqrt(epsilon(1.0d0))


! Initialization

        do k = 1, n
           lstrow(k) = 0
        enddo

! Proceed row by row:

        do k = 1, n
! 
! Set up the array lstrow that will contains the address that 
! link row k in sparse form.
! zero out row entries which correspond to allowed nonzeros in LU
!
            do j =  ilu(k),  ilu(k+1)-1
            lstrow( abs(ilu(j)))  = j
                lu(j) = 0.0d0
        enddo
!
! Set the diagonal entry:
!
            lstrow(k) =  k
            lu(k) =  0.0d0
!
! Set entries of row corresponding to original nonzeros of A to
! those values from A:
!
            do j = r_point(k),r_point(k+1)-1
        k1 = c_ind(j)
        if (lstrow(k1).ne. 0) then
            lu(lstrow(k1)) = a(j)
        endif
        enddo
!
! At this point, row(:) contains the k-th row of LU, both upper and 
! lower triangular parts.  All but possibly the first (k,1) entry
! will need to have previous updates applied.
!
! Process the k-th row of LU, by updating it with the multiplier
! in (k,i) times U(i,i+1:n):
!
        do i =  ilu(k), ilup(k)-1
!
! jli is the column index of an entry in L in row k, and
! li is the corresponding value in that (k,jli) position.
! This becomes the multiplier lu(i) = L(k,jli) on this stage of
! processing row k:
!
                jli =  ilu(i)
            if (jli .gt. 0) then
!               only element in R could be fill in
                    li =  lu(i)*lu(jli)
                    lu(i) =  li
                    do j = ilup(jli),  ilu(jli+1)-1
                        jcol      =  ilu(j)
                if (jcol.lt.0) then
                jcol = -jcol
                if (lstrow(jcol) .ne. 0) then
                    lu(lstrow(jcol))=lu(lstrow(jcol))- li*lu(j)
                endif
                        endif
            enddo
            else
!       all elements can be fillin
            jli = -jli
            li =  lu(i)*lu(jli)
                    lu(i) =  li
                    do j = ilup(jli),  ilu(jli+1)-1
                        jcol      =  abs(ilu(j))
                if (lstrow(jcol) .ne. 0) then
                    lu(lstrow(jcol)) = lu(lstrow(jcol))-li*lu(j)
                endif
            enddo
            endif
        enddo
!
!  At this point, all the updates to row k are complete.  The
!  diagonal entry is the corresponding one of L, and will be
!  stored in inverted form to avoid divisions in further processing
!  and in the triangular solves with L.  Note that the test is
!  against exact zero, and should be replaced with a tolerance.
!
            if (abs(lu(k)) .le. SMALL) then
!                ierr  =  k
!                return
                 lu(k) = sign(SMALL,lu(k))
            endif
            dk = 1.0d0/lu(k)
            lu(k) = dk
!
! Now scale the upper triangular entries in row k using the
! inverted diagonal entry:
!
!         do 100 j = ilup(k), ilu(k+1)-1
!            lu(j) = row( ilu(j) )
!            lu(j) = row( ilu(j) ) * dk
! 100     continue
!
! Clear the vector lstrow before starting the next row
!
          do j = ilu(k), ilu(k+1)-1
              lstrow(abs(ilu(j))) = 0
          enddo
          lstrow(k) = 0
! Go to next row.
!
    enddo
!
! Extract R out,
!
    lp = n+2
    istart = ilu(1)
    do i = 1, n
        do k = istart, ilup(i)-1
        if (ilu(k).lt.0) then
            ilu(lp) = -ilu(k)
            lu(lp) = lu(k)
            lp = lp + 1
        endif
        enddo
        istart = ilup(i)
        ilup(i) = lp
        do k = istart, ilu(i+1)-1
                if (ilu(k).lt.0) then
                    ilu(lp) = -ilu(k)
            lu(lp) = lu(k)
                    lp = lp + 1
                endif
        enddo
        istart = ilu(i+1)
        ilu(i+1) = lp
    enddo
!
    ierr  = 0
        return
        end
