      program filldiag
      implicit none
!----------------------------------------------------------------------
!
!    This program can be plugged into the place of "driver"
!    in SPLIB, in order to fill in the diagonal entries of
!    a H/B matrix.  It does this by allowing space for missing
!    diagonal entries in readmt3, and then calling matchk to
!    explicitly insert a 0.0 (or any value specified by INSERT
!    in matchk.f) wherever a diagonal entry is missing.
!
!     Randall Bramley
!     Department of Computer Science
!     Indiana University
!     Phone: (812) 855-7790
!     email: bramley@cs.indiana.edu
!     Fri Dec 20 10:46:22 EST 1996
!
!----------------------------------------------------------------------
!
!     ------------------------
!     Total memory for arrays:
!     ------------------------
      integer     mtot
      parameter ( mtot =  48 000 000)                                  
      integer  work
      common /memory/ work(mtot)                                    
      include 'include/memory.inc'

!     --------------------------------------------------
!     Number of integer words per double precision word:
!     --------------------------------------------------
      include 'include/intdbl.inc'
      include 'include/symbols.inc'

!     --------------------------------
!     Double precision array pointers:
!     --------------------------------
      integer a, x, b, xs, g, Dl, Dr

!     -------------------------
!     Double precision scalars:
!     -------------------------
      double precision :: tol, rpcprm
      double precision :: err

!     ----------------------------
!     Function programs from BLAS:
!     ----------------------------
      double precision :: ddot,dasum,dnrm2
      external ddot,dasum,dnrm2

!     -------------------------------------
!     Function programs from memory manager:
!     -------------------------------------
      integer addtbl, loctbl, sizeof, storof, getnext, memleft
      logical istbl
      external addtbl, loctbl, sizeof, storof, istbl, getnext, memleft

!     -----------------------
!     Integer array pointers:
!     -----------------------
      integer rwptr, colind
      integer next, remain

!     ---------
!     Integers:
!     ---------
      integer ierr, maxits, iparms(2), n, nnz
      integer i, k, mneed
      integer job, nrhs, ipcprm
      integer pcmeth, solmeth, krylov, ndiag
      integer digits
      integer fmtd
      integer scale, iounit, ifmt

!     ------------------
!     Logical variables:
!     ------------------
      logical chkdiag
      logical lsol, lrhs, lgus, reltv, redo, firstmat
      data firstmat /.true./

!     --------------------
!     Character variables:
!     --------------------
      character type*3, guesol*2
      character title*72, key*8
      character solver*5, precond*4
      character from*8

!     --------------------------
!     Instrumentation variables:
!     --------------------------
      integer iunit(16), instlvl
      common /truesol/ xstar
      double precision :: xstar(1)

!-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
!-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
!      Beginning of Executable Statements:
!-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
!-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

!     ----------------------------------------------
!     Input file for matrix in Harwell/Boeing format
!     ----------------------------------------------
      iunit(3) = 4

!     ----------------------------------
!     Input file with control parameters
!     ----------------------------------
      iunit(2) = 5

!     ----------------------------------------------------
!     Output file with echoed input and summary of run(s).
!     ----------------------------------------------------

      iunit(1) = 60

!     ------------------------------------------------
!     Other unit number somewhat arbitrarily assigned:
!     ------------------------------------------------
      do i = 4,20
         iunit(i) = i+3
      end do 

!     -----------------------------------------
!     Open required files for input and results
!     -----------------------------------------
      open(iunit(2),file='methods')
      open(iunit(1),file='results')

!     ----------------------------------------------------------------
!     Initialize memory management.  Have it write to 'results' file.
!     ----------------------------------------------------------------
      call initbl(iunit(1),mtot)

!     -------------------------------------------------
!     Set pointer to next available location in work():
!     -------------------------------------------------
        next = 1
        remain = mtot

!*****************************************************************************
!****** Big "go to" loop over problem solution/preconditioner specifications:*
!*****************************************************************************

!     ------------------------------------------------------
!     Try to read right hand side and solution vector.
!     If not supplied, create the right hand side vector b.
!     ------------------------------------------------------
      job = 3

!     -----------------------------------------------------------------------
!     Read matrix and vectors, and set up pointers.  Note that readmt3 will
!     insert extra positions after the colind array, to provide for 
!     the possible expansion of data structure by inserting missing diagonal
!     entries.
!     -----------------------------------------------------------------------
!       ------------------------------------------------------------
!       Force matrix data structure to have diagonal entries present
!       ------------------------------------------------------------
        chkdiag = .true.
        open(iunit(3),file='matrix',form='formatted')
        call readmt3(mtot  ,job,iunit(3),a,colind,rwptr,b,xs,g,nrhs,    &
              work(next),mneed,guesol,n,n,nnz,title,key,type,ierr,      &
              chkdiag)
        if (ierr .ne. 0) then
             write(iunit(1),5000) ierr
             write(iunit(1),5010) mneed
             stop
        end if 
        lrhs = (job .ge. 3)
        lsol = (guesol(2:2) .eq. 'X' .or. guesol(2:2) .eq. 'x')
        lgus = (guesol(1:1) .eq. 'G' .or. guesol(1:1) .eq. 'g')

!       --------------------------------------------------------------
!       Now update next and remain to reflect storage used in readmt3.
!       --------------------------------------------------------------
        from = 'readmt'
        k = addtbl('rwptr', n+1, 'integer', from)
        k = addtbl('colind', nnz, 'integer', from)
        k = addtbl('a', nnz, 'real', from)
        if (lrhs) k = addtbl('b', n*nrhs, 'real', from)
        if (lgus) k = addtbl('g', n*nrhs, 'real', from)
        if (lsol) k = addtbl('xs', n*nrhs, 'real', from)
        next = getnext()
        remain = memleft()

!       --------------------------------------------------------------
!       Call matrix checker to insert missing diagonal entries, and to
!       sort rows into increasing column order:
!       --------------------------------------------------------------
        if (remain .lt. n) then
             write(iunit(1),5050) next
             stop
        end if
        call matchk(work(a),work(colind),work(rwptr),n,work(next), ndiag)
        ifmt = 6
        job = 3
        iounit = 14
        open(14, file = 'matrix.new')
        call prtmt(n,n,work(a),work(colind),work(rwptr),work(b),guesol, &
                   title,key,type, ifmt,job,iounit)
        stop


!========================================================================
!           Format Statements
!========================================================================

 6020  format('Order of matrix = ',i6,/,'Number of nonzeros ',i8)
 6040  format('Tolerance on residual = ',1pe10.4)
 6050  format('Iterative method used: ',a8,' with ',a8, ' preconditioning')
 6060  format('Levels of fill allowed in precond: ',i3)
 6070  format(/,40('--'),/,'Problem title: ',a71)
 6080  format('Problem key: ',a8)
 6090  format('Acceleration parameter: ',f8.2)

 2010  format('Final error 2-norm: ',1pe12.5)

 5000  format(' *** Error in readmt ; ierr = ',i4)
 5010  format(' *** Error in readmt ; mneed = ',i7)
 5020  format(' *** Error in driver; ran out of storage to set up x.',      &
              '     next = ',i7)
 5030  format(' *** Error in driver; ran out of storage to set up b.',      &
              '     next = ',i7)
 5040  format(' *** Error in driver; ran out of storage to set up r.',      &
              '     next = ',i7)
 5050  format(' *** Error in driver; insuff storage for matchk',        &
              '     next = ',i7)

	end
!----------------------------------------------------------------------
