!======================================================================== 
      subroutine rowinc(num,q,adata)
      use kinds
      implicit none
!======================================================================== 
!
!  Shell sort, with hardwired increments.  The algorithm for
!  sorting entries in q(0:n-1) is as follows:
!----------------------------------------------------------------
!  inc = initialinc(n)
!  while inc >= 1
!     for i = inc to n-1
!         j = i
!         x = q(i)
!         while j >= inc and q(j-inc) > x
!            q(j) = q(j-inc)
!            j    = j-inc
!         end while 
!         q(j) = x
!     end for
!     inc = nextinc(inc,n)
!  end while
!----------------------------------------------------------------
!
!  However, the code implemented here is for q(1:n), in keeping with
!  the Fortran convention of indexing from 1.
!  The increments used are 1, 4, 13, 40, 121, ..., (3**i - 1)/2, ...
!  In this case, nextinc(inc,n) = (inc-1)/3.  Usually shellsort
!  would have the largest increment the largest integer of the form
!  (3**i - 1)/2 that is less than n, but here it is fixed at 121
!  because most sparse matrices have 121 or fewer nonzero entries
!  per row.  If this routine is expanded for a complete sparse
!  factorization routine, or if a large number of levels of fill is
!  allowed, then possibly it should be replaced with more efficient
!  sorting.  This routine differs from srtr in symbfac because it
!  also sorts the data field given by adata(*).
!
!     Randall Bramley
!     Department of Computer Science
!     Indiana University
!     email: bramley@cs.indiana.edu
!     Tue May 17 11:37:38 EST 1994
!
!======================================================================== 

      integer       :: num, q(num)
      real(real8)   :: adata(num), t
      integer       ::  iinc(5), key, icn, ih, ii, i, j, jj

      data iinc/1,4,13,40,121/

      if   (num  ==  0) then
            icn   =  0
      elseif   (num  <   14) then
                  icn   =  1
            elseif   (num  <   41) then
                  icn   =  2
                  elseif   (num  <   122) then
                              icn   =  3
                        elseif   (num  <   365) then
                                    icn   =  4
                              else
                                    icn   =  5
      end if
      do ii = 1, icn
            ih = iinc(icn + 1 - ii)
            do j = ih+1, num
                  i = j-ih
                  key = q(j)
                  t   = adata(j)
                  do jj = 1, j-ih, ih
                        if (key  >=  q(i)) exit
                        q(i + ih)     = q(i)
                        adata(i + ih) = adata(i)
                        i             = i - ih
                  end do
                  q(i + ih) = key
                  adata(i + ih) = t
            end do  
      end do  

      return
      end
